﻿// ***********************************************************************************
// Garbage Collector
// ***********************************************************************************
// Created by Sally Whitemane on VAM Hub
// Licensed under CC0 - https://creativecommons.org/publicdomain/zero/1.0/
using System;
using UnityEngine.UI;

namespace Sally { 
	public class GarbageCollector : MVRScript
    {
        private static MemoryOptimizer mem = MemoryOptimizer.singleton;
        private static String infoText =
        "<b>Garbage Collector</b>\n" +
        "VAMs Memory Optimization usable via trigger.\n\n" +
        "This plugin is mostly intended for <b>creators</b> of memory heavy scene that dynamically load new content at some point in time. Usually story or dances scenes - not simple static scenes.\n\n" +
        "<b>Usage</b>\n" +
        "1. Create a trigger and set the Receiver to this Plugin\n" +
        "2. Set Receiver Target to call: <color=#66F><b>OptimizeMemory</b></color>\n\n" +
        "<color=#F22><b>WARNING:\n" +
        "Unused assets like skins, clothing, hair and morphs will be unloaded!\n" +
        "Please make users aware of this in your scene description!</b></color>\n\n" +
        "<color=#2F2><b>ADVATAGE:</b></color>\n" +
        "Can prevent VAM from crashing. Especially on low memory systems with 16 GB or lower.\n\n" +
        "When <i><b>chain loading a lot new content</b></i> into memory without changing scenes VAM never frees certain memory. Running out of free memory will then results in a crash. To prevent this creators can use this Plugin to trigger the memory optimization manually with the scene logic. This should be done a strategic good time. For example when a new location is loaded into the scene.\n\n" +
        "If the <b>enabled</b> flag is <b>false</b> calls to <color=#66F><b>OptimizeMemory</b></color> will be ignored. This can be used for a <b>UIToggle</b> in the scene UI to let the user decide whether he wants the memory optimized.\n\n";
        private static String memoryText = "< Results will be visible here when called >";
        private JSONStorableBool jbLogMsg = new JSONStorableBool("Show result in message log", false);
        private JSONStorableString text = new JSONStorableString("Text", infoText + memoryText);

        public override void Init()
        {
            try
            {
                UIDynamic uiText = CreateTextField(text);
                uiText.height = 1000f;
                uiText.GetComponent<LayoutElement>().minWidth = 1060;
                CreateButton("Optimize").button.onClick.AddListener(Optimize);
                CreateSpacer(true).height = 1000f;
                CreateToggle(jbLogMsg,true);

                JSONStorableAction OptimizeMemory = new JSONStorableAction("OptimizeMemory", () =>
                {
                    Optimize();
                });
                RegisterAction(OptimizeMemory);
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }
        private void Optimize()
        {
            if (!enabled)
                return; // Do nothing if Plugin disabled

            // Get approximation of the number of bytes allocated in managed memory before optimization
            var before = GC.GetTotalMemory(false);
            // Call VAMs 'official' Memory optimization
            // It will unload unused asset and then run garbage collection
            mem.TriggerOptimize();
            // Get approximation of the number of bytes allocated in managed memory AFTER optimization
            var after = GC.GetTotalMemory(true);
            memoryText = "bytes freed: " + (before - after).ToString("N0") + "  before: " + before.ToString("N0") + "  after: " + after.ToString("N0");
            if (jbLogMsg.val)
                SuperController.LogMessage(memoryText);
            // Update Plugin text
            text.val = infoText + memoryText;
        }

        // accuracy questionable, better look at official VAM UI
        // timing of the report may be relevant
        /*
        private void UpdateMemoryText()
        {
            memoryText =
            mem.HeapSizeInGBText + " Heap\n" +
            mem.SystemMemorySizeInGBText + " System\n" +
            mem.WorkingSetSizeInGBText + " Working Set  " + mem.PeakWorkingSetSizeInGBText + " Peak\n" +
            mem.PageFileUsageInGBText + " Page File  " + mem.PeakPageFileUsageInGBText + " Peak\n";
            text.val = infoText + memoryText;
        }
        */
    }
}